﻿// #define DrawBox

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Drawing;

namespace howto_draw_equations
{
    // Draw vertical bars beside the contents.
    class BarEquation : Equation
    {
        // Bar styles.
        public enum BarStyles
        {
            None,
            Bar,
            Bracket,
            Brace,
            PointyBracket,
            Parenthesis,
        }

        // The contents.
        private Equation Contents;

        // The bar style.
        private BarStyles LeftBarStyle, RightBarStyle;

        // Extra space around the content.
        public float MarginX = 5;
        public float MarginY = 5;

        // Width of bars.
        private const float BracketWidth = 8;
        private const float BracesWidth = 6;
        private const float PointyBracketWidthFraction = 0.1f;
        private const float ParenthesisWidthFraction = 0.1f;

        // Initialize the contents.
        public BarEquation(Equation contents, BarStyles left_bar_style, BarStyles right_bar_style)
        {
            Contents = contents;
            LeftBarStyle = left_bar_style;
            RightBarStyle = right_bar_style;
        }

        // Set font sizes for sub-equations.
        public override void SetFontSizes(float font_size)
        {
            FontSize = font_size;
            Contents.SetFontSizes(font_size);
        }

        // Return the equation's size.
        public override SizeF GetSize(Graphics gr, Font font)
        {
            // Get the content's height.
            SizeF item_size = Contents.GetSize(gr, font);

            // Add vertical space.
            item_size.Height += 2 * MarginY;

            // Add room for the bars.
            item_size.Width +=
                2 * MarginX +
                GetBarWidth(gr, font, item_size.Height, LeftBarStyle) +
                GetBarWidth(gr, font, item_size.Height, RightBarStyle);

            return item_size;
        }

        // Return the size of a bar.
        private float GetBarWidth(Graphics gr, Font font, float our_height, BarStyles bar_style)
        {
            switch (bar_style)
            {
                case BarStyles.Bar:
                    return 0;
                case BarStyles.Brace:
                    return 2 * BracesWidth;
                case BarStyles.Bracket:
                    return BracketWidth;
                case BarStyles.PointyBracket:
                    return our_height * PointyBracketWidthFraction;
                case BarStyles.Parenthesis:
                    return our_height * ParenthesisWidthFraction;
                case BarStyles.None:
                    return 0;
                default:
                    throw new ArgumentOutOfRangeException("bar_style",
                        "Unknown BarStyles value " + LeftBarStyle.ToString());
            }
        }

        // Draw the equation.
        public override void Draw(Graphics gr, Font font, Pen pen, Brush brush, float x, float y)
        {
            // Get our size.
            SizeF our_size = GetSize(gr, font);

#if DrawBox
            using (Pen dashed_pen = new Pen(Color.Orange, 1))
            {
                dashed_pen.DashStyle = System.Drawing.Drawing2D.DashStyle.Dash;
                gr.DrawRectangle(dashed_pen, x, y, our_size.Width, our_size.Height);
            }
#endif

            // Draw the bars.
            DrawLeftBar(our_size, x, y, gr, font, pen);
            DrawRightBar(our_size, x, y, gr, font, pen);

            // Draw the contents.
            float contents_x = x + MarginX + GetBarWidth(gr, font, our_size.Height, LeftBarStyle);
            Contents.Draw(gr, font, pen, brush, contents_x, y + MarginY);
        }

        // Draw a left bar.
        private void DrawLeftBar(SizeF our_size, float x, float y, Graphics gr, Font font, Pen pen)
        {
            x += MarginX;
            switch (LeftBarStyle)
            {
                case BarStyles.Bar:
                    gr.DrawLine(pen, x, y, x, y + our_size.Height);
                    break;
                case BarStyles.Bracket:
                    PointF[] bracket_pts = 
                    {
                        new PointF(x + BracketWidth, y),
                        new PointF(x, y),
                        new PointF(x, y + our_size.Height),
                        new PointF(x + BracketWidth, y + our_size.Height),
                    };
                    gr.DrawLines(pen, bracket_pts);
                    break;
                case BarStyles.Brace:
                    PointF[] brace_pts = 
                    {
                        new PointF(x + 2 * BracesWidth, y),
                        new PointF(x + BracesWidth, y + 2 * BracesWidth),
                        new PointF(x + BracesWidth, y + our_size.Height / 2 - 2 * BracesWidth),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x + BracesWidth, y + our_size.Height / 2 + 2 * BracesWidth),
                        new PointF(x + BracesWidth, y + our_size.Height - 2 * BracesWidth),
                        new PointF(x + 2 * BracesWidth, y + our_size.Height),
                    };
                    gr.DrawCurve(pen, brace_pts);
                    break;
                case BarStyles.PointyBracket:
                    float pointy_bracket_width = PointyBracketWidthFraction * our_size.Height;
                    PointF[] point_bracket_pts = 
                    {
                        new PointF(x + pointy_bracket_width, y),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x + pointy_bracket_width, y + our_size.Height),
                    };
                    gr.DrawLines(pen, point_bracket_pts);
                    break;
                case BarStyles.Parenthesis:
                    float parenthesis_width = PointyBracketWidthFraction * our_size.Height;
                    PointF[] parenthesis_pts = 
                    {
                        new PointF(x + parenthesis_width, y),
                        new PointF(x + parenthesis_width / 2, y + 2 * parenthesis_width),
                        new PointF(x + parenthesis_width / 2, y + our_size.Height - 2 * parenthesis_width),
                        new PointF(x + parenthesis_width, y + our_size.Height),
                    };
                    gr.DrawCurve(pen, parenthesis_pts);
                    break;
                case BarStyles.None:
                    break;
                default:
                    throw new ArgumentOutOfRangeException("LeftBarStyle",
                        "Unknown BarStyles value " + LeftBarStyle.ToString());
            }
        }

        // Draw a right bar.
        private void DrawRightBar(SizeF our_size, float x, float y, Graphics gr, Font font, Pen pen)
        {
            x += our_size.Width - MarginX;
            switch (RightBarStyle)
            {
                case BarStyles.Bar:
                    gr.DrawLine(pen, x, y, x, y + our_size.Height);
                    break;
                case BarStyles.Bracket:
                    PointF[] bracket_pts = 
                    {
                        new PointF(x - BracketWidth, y),
                        new PointF(x, y),
                        new PointF(x, y + our_size.Height),
                        new PointF(x - BracketWidth, y + our_size.Height),
                    };
                    gr.DrawLines(pen, bracket_pts);
                    break;
                case BarStyles.Brace:
                    PointF[] brace_pts = 
                    {
                        new PointF(x - 2 * BracesWidth, y),
                        new PointF(x - BracesWidth, y + 2 * BracesWidth),
                        new PointF(x - BracesWidth, y + our_size.Height / 2 - 2 * BracesWidth),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x - BracesWidth, y + our_size.Height / 2 + 2 * BracesWidth),
                        new PointF(x - BracesWidth, y + our_size.Height - 2 * BracesWidth),
                        new PointF(x - 2 * BracesWidth, y + our_size.Height),
                    };
                    gr.DrawCurve(pen, brace_pts);
                    break;
                case BarStyles.PointyBracket:
                    float pointy_bracket_width = PointyBracketWidthFraction * our_size.Height;
                    PointF[] point_bracket_pts = 
                    {
                        new PointF(x - pointy_bracket_width, y),
                        new PointF(x, y + our_size.Height / 2),
                        new PointF(x - pointy_bracket_width, y + our_size.Height),
                    };
                    gr.DrawLines(pen, point_bracket_pts);
                    break;
                case BarStyles.Parenthesis:
                    float parenthesis_width = PointyBracketWidthFraction * our_size.Height;
                    PointF[] parenthesis_pts = 
                    {
                        new PointF(x - parenthesis_width, y),
                        new PointF(x - parenthesis_width / 2, y + 2 * parenthesis_width),
                        new PointF(x - parenthesis_width / 2, y + our_size.Height - 2 * parenthesis_width),
                        new PointF(x - parenthesis_width, y + our_size.Height),
                    };
                    gr.DrawCurve(pen, parenthesis_pts);
                    break;
                case BarStyles.None:
                    break;
                default:
                    throw new ArgumentOutOfRangeException("RightBarStyle",
                        "Unknown BarStyles value " + RightBarStyle.ToString());
            }
        }
    }
}
